@@ -6,23 +6,12 @@
 #include "util/FFstrbuf.h"
 #include "common/library.h"
 
-#define FF_DBUS_TIMEOUT_MILLISECONDS 35
-
-#define FF_DBUS_ITER_CONTINUE(dbus, iterator) \
-    { \
-        if(!(dbus)->lib->ffdbus_message_iter_has_next(iterator)) \
-            break; \
-        (dbus)->lib->ffdbus_message_iter_next(iterator); \
-        continue; \
-    }
-
 typedef struct FFDBusLibrary
 {
     FF_LIBRARY_SYMBOL(dbus_bus_get)
     FF_LIBRARY_SYMBOL(dbus_message_new_method_call)
+    FF_LIBRARY_SYMBOL(dbus_message_append_args)
     FF_LIBRARY_SYMBOL(dbus_message_iter_init)
-    FF_LIBRARY_SYMBOL(dbus_message_iter_init_append)
-    FF_LIBRARY_SYMBOL(dbus_message_iter_append_basic)
     FF_LIBRARY_SYMBOL(dbus_message_iter_get_arg_type)
     FF_LIBRARY_SYMBOL(dbus_message_iter_get_basic)
     FF_LIBRARY_SYMBOL(dbus_message_iter_recurse)
@@ -30,10 +19,6 @@ typedef struct FFDBusLibrary
     FF_LIBRARY_SYMBOL(dbus_message_iter_next)
     FF_LIBRARY_SYMBOL(dbus_message_unref)
     FF_LIBRARY_SYMBOL(dbus_connection_send_with_reply_and_block)
-    FF_LIBRARY_SYMBOL(dbus_connection_flush)
-    FF_LIBRARY_SYMBOL(dbus_pending_call_block)
-    FF_LIBRARY_SYMBOL(dbus_pending_call_steal_reply)
-    FF_LIBRARY_SYMBOL(dbus_pending_call_unref)
 } FFDBusLibrary;
 
 typedef struct FFDBusData
@@ -43,11 +28,18 @@ typedef struct FFDBusData
 } FFDBusData;
 
 const char* ffDBusLoadData(DBusBusType busType, FFDBusData* data); //Returns an error message or NULL on success
-bool ffDBusGetValue(FFDBusData* dbus, DBusMessageIter* iter, FFstrbuf* result);
+bool ffDBusGetString(FFDBusData* dbus, DBusMessageIter* iter, FFstrbuf* result);
 bool ffDBusGetBool(FFDBusData* dbus, DBusMessageIter* iter, bool* result);
-bool ffDBusGetByte(FFDBusData* dbus, DBusMessageIter* iter, uint8_t* result);
-DBusMessage* ffDBusGetMethodReply(FFDBusData* dbus, const char* busName, const char* objectPath, const char* interface, const char* method);
+bool ffDBusGetUint(FFDBusData* dbus, DBusMessageIter* iter, uint32_t* result);
+DBusMessage* ffDBusGetMethodReply(FFDBusData* dbus, const char* busName, const char* objectPath, const char* interface, const char* method, const char* arg1, const char* arg2);
 DBusMessage* ffDBusGetProperty(FFDBusData* dbus, const char* busName, const char* objectPath, const char* interface, const char* property);
-void ffDBusGetPropertyString(FFDBusData* dbus, const char* busName, const char* objectPath, const char* interface, const char* property, FFstrbuf* result);
+bool ffDBusGetPropertyString(FFDBusData* dbus, const char* busName, const char* objectPath, const char* interface, const char* property, FFstrbuf* result);
+bool ffDBusGetInt(FFDBusData* dbus, DBusMessageIter* iter, int32_t* result);
+bool ffDBusGetPropertyUint(FFDBusData* dbus, const char* busName, const char* objectPath, const char* interface, const char* property, uint32_t* result);
+
+static inline DBusMessage* ffDBusGetAllProperties(FFDBusData* dbus, const char* busName, const char* objectPath, const char* interface)
+{
+    return ffDBusGetMethodReply(dbus, busName, objectPath, "org.freedesktop.DBus.Properties", "GetAll", interface, NULL);
+}
 
 #endif // FF_HAVE_DBUS
